Complete XML structure.
[pspp] / dump.c
diff --git a/dump.c b/dump.c
index bc3fe39388a26c6b65f103c655ac7320e949f618..73935a73374faf7df3695a23436ad8c2506385ba 100644 (file)
--- a/dump.c
+++ b/dump.c
@@ -252,10 +252,15 @@ dump_value_31(FILE *stream)
             {
               /* We only have one SPV file for this version (with many
                  tables). */
-              match_u32_assert(0x200);
-              match_u32_assert(0x1000000);
-              match_u32_assert(0);
-              match_byte_assert(0);
+              match_byte(0);
+              if (!match_u32(1))
+                match_u32_assert(2);
+              match_byte(0);
+              match_byte(0);
+              if (!match_u32(0) && !match_u32(1) && !match_u32(2) && !match_u32(3) && !match_u32(4) && !match_u32(5) && !match_u32(6) && !match_u32(7) && !match_u32(8) && !match_u32(9))
+                match_u32_assert(10);
+              match_byte(0);
+              match_byte(0);
               return;
             }
 
@@ -298,15 +303,16 @@ dump_value_31(FILE *stream)
             match_byte_assert(0x58);
           if (match_byte(0x31))
             {
-              /* All the examples in the corpus, all from one SPV file, are
-                 exactly like this. */
+              /* Only two SPV files have anything like this, so it's hard to
+                 generalize. */
               match_u32_assert(0);
               match_u32_assert(0);
               match_u32_assert(0);
               match_u32_assert(0);
               match_byte_assert(1);
               match_byte_assert(0);
-              match_byte_assert(8);
+              if (!match_byte(8) && !match_byte(1))
+                match_byte_assert(2);
               match_byte_assert(0);
               match_byte_assert(8);
               match_byte_assert(0);
@@ -329,10 +335,11 @@ dump_value_31(FILE *stream)
       else if (match_u32 (2))
         {
           fprintf(stream, "(special 2)");
+          if (!match_byte(0))
+            match_byte_assert(2);
           match_byte_assert(0);
-          match_byte_assert(0);
-          if (!match_u32 (2))
-            match_u32_assert(1);
+          if (!match_u32 (2) && !match_u32(1))
+            match_u32_assert(3);
           dump_nested_string(); /* Our corpus doesn't contain any examples with strings though. */
         }
       else
@@ -445,7 +452,7 @@ dump_value(FILE *stream, int level, bool match1)
       char *var, *vallab;
       double value;
 
-      match_byte_assert (0x58);
+      dump_value_31 (stream);
       format = get_u32 ();
       value = get_double ();
       var = get_string ();
@@ -548,7 +555,7 @@ check_permutation(int *a, int n, const char *name)
 }
 
 static void
-dump_category(int level, int *indexes, int *n_indexes)
+dump_category(int level, int *indexes, int *n_indexes, int max_indexes)
 {
   for (int i = 0; i <= level; i++)
     fprintf (stdout, "    ");
@@ -579,6 +586,11 @@ dump_category(int level, int *indexes, int *n_indexes)
           fprintf(stderr, "index not -1 but subcategories\n");
           exit(1);
         }
+      if (*n_indexes >= max_indexes)
+        {
+          fprintf(stderr, "too many categories (increase max_indexes)\n");
+          exit(1);
+        }
       indexes[(*n_indexes)++] = indx;
     }
   if (n_categories == 0)
@@ -588,7 +600,7 @@ dump_category(int level, int *indexes, int *n_indexes)
       fprintf (stdout, "<category-index>%d</category-index>\n", indx);
     }
   for (int i = 0; i < n_categories; i++)
-    dump_category (level + 1, indexes, n_indexes);
+    dump_category (level + 1, indexes, n_indexes, max_indexes);
   for (int i = 0; i <= level; i++)
     fprintf (stdout, "    ");
   printf ("</category>\n");
@@ -602,7 +614,7 @@ dump_dim(int indx)
   printf ("<dimension index=\"%d\">\n", indx);
   dump_value (stdout, 0, false);
 
/* This byte is usually 0x02 but 0x00 and 0x75 (!) have also been spotted. */
   pos++;
 
   if (!match_byte(0) && !match_byte(1))
@@ -614,13 +626,14 @@ dump_dim(int indx)
   if (!match_byte(0))
     match_byte_assert(1);
   match_byte_assert(1);
-  match_u32_assert(indx);
+  if (!match_u32(UINT32_MAX))
+    match_u32_assert(indx);
   n_categories = get_u32();
 
-  int indexes[1024];
+  int indexes[2048];
   int n_indexes = 0;
   for (int i = 0; i < n_categories; i++)
-    dump_category (0, indexes, &n_indexes);
+    dump_category (0, indexes, &n_indexes, sizeof indexes / sizeof *indexes);
   check_permutation(indexes, n_indexes, "categories");
 
   fprintf (stdout, "</dimension>\n");
@@ -655,6 +668,8 @@ dump_data(void)
     {
       printf ("    <datum index=\"%d\">\n", get_u32());
       match_u32_assert(0);
+      if (version == 1)
+        match_byte(0);
       dump_value(stdout, 1, false);
       fprintf (stdout, "    </datum>\n");
     }
@@ -724,12 +739,12 @@ dump_fonts(void)
       printf(" font=\"%s\"", get_string());
       match_byte_assert(0);
       match_byte_assert(0);
-      if (!match_byte(0x40) && !match_byte(0x20) && !match_byte(0x80) && !match_byte(0x10))
+      if (!match_byte(0x40) && !match_byte(0x20) && !match_byte(0x80) && !match_byte(0x10) && !match_byte(0x70))
         match_byte_assert(0x50);
       if (!match_byte(0x41))
         match_byte_assert(0x51);
-      if (!match_u32(0))
-        match_u32_assert(1);
+      if (!match_u32(0) && !match_u32(1))
+        match_u32_assert(2);
       match_byte_assert(0);
 
       /* OK, this seems really unlikely to be totally correct, but it matches my corpus... */
@@ -749,10 +764,12 @@ dump_fonts(void)
           /* These seem unlikely to be correct too. */
           if (i != 3)
             {
-              match_u32_assert(8);
-              if (!match_u32(10))
-                match_u32_assert(11);
-              match_u32_assert(1);
+              if (!match_u32(8))
+                match_u32_assert(5);
+              if (!match_u32(10) && !match_u32(11) && !match_u32(5))
+                match_u32_assert(9);
+              if (!match_u32(0))
+                match_u32_assert(1);
             }
           else
             {
@@ -780,6 +797,10 @@ dump_fonts(void)
     pos += 117;
   else if (match_u32(142))
     pos += 142;
+  else if (match_u32(143))
+    pos += 143;
+  else if (match_u32(150))
+    pos += 150;
   else
     {
       match_u32_assert(16);
@@ -809,17 +830,25 @@ dump_fonts(void)
   else
     match_u32_assert(UINT32_MAX);
   if (match_byte('.'))
-    match_byte_assert(',');
+    {
+      if (!match_byte(','))
+        match_byte_assert(' ');
+    }
   else
     {
       match_byte_assert(',');
-      if (!match_byte('.'))
-        match_byte_assert(' ');
+      if (!match_byte('.') && !match_byte(' '))
+        match_byte_assert(0);
     }
-  match_u32_assert(5);
-  for (int i = 0; i < 5; i++)
-    get_string();
-  pos += get_u32();
+  if (match_u32(5))
+    {
+      for (int i = 0; i < 5; i++)
+        get_string();
+    }
+  else
+    match_u32_assert(0);
+  int skip = get_u32();
+  pos += skip;
 }
 
 int
@@ -602,7 +614,7 @@ dump_dim(int indx)
   dump_value (stdout, 0, false);
 
-  /* This byte is usually 0x02 but 0x00 and 0x75 (!) have also been spotted. */
+  /* This byte is usually 0x02 but many other values have been spotted. */
   pos++;
 